@@ -1,35 +1,39 @@
 /* ebrowse.c --- parsing files for the ebrowse C++ browser
 
-   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-	         2000, 2001
-      Free Software Foundation Inc.
+Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+              Free Software Foundation, Inc.
 
-   Author: Gerd Moellmann <gerd@gnu.org>
-   Maintainer: FSF
-   
-   This file is part of GNU Emacs.
+This file is part of GNU Emacs.
 
-   GNU Emacs is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+GNU Emacs is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-   GNU Emacs is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+GNU Emacs is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Emacs; see the file COPYING.  If not, write to
-   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <stdio.h>
+
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+
 #include <ctype.h>
 #include <assert.h>
 #include "getopt.h"
@@ -400,7 +404,7 @@ int inbuffer_size;
 
 #define BUFFER_POS() (in - inbuffer)
 
-/* If current lookahead is CSTRING, the following points to the 
+/* If current lookahead is CSTRING, the following points to the
    first character in the string constant.  Used for recognizing
    extern "C".  */
 
@@ -529,6 +533,8 @@ struct sym *parse_qualified_ident_or_type P_ ((char **));
 void parse_qualified_param_ident_or_type P_ ((char **));
 int globals P_ ((int));
 void yyerror P_ ((char *, char *));
+void usage P_ ((int)) NO_RETURN;
+void version P_ (()) NO_RETURN;
 
 
 
@@ -560,7 +566,7 @@ xmalloc (nbytes)
   if (p == NULL)
     {
       yyerror ("out of memory", NULL);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   return p;
 }
@@ -577,23 +583,12 @@ xrealloc (p, sz)
   if (p == NULL)
     {
       yyerror ("out of memory", NULL);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   return p;
 }
 
 
-/* Like free but always check for null pointers..  */
-
-void
-xfree (p)
-     void *p;
-{
-  if (p)
-    free (p);
-}
-
-
 /* Like strdup, but print an error and exit if not enough memory is
    available..  If S is null, return null.  */
 
@@ -644,7 +639,10 @@ add_sym (name, nested_in_class)
   h %= TABLE_SIZE;
 
   for (sym = class_table[h]; sym; sym = sym->next)
-    if (streq (name, sym->name) && sym->namesp == scope)
+    if (streq (name, sym->name)
+	&& ((!sym->namesp && !scope)
+	    || (sym->namesp && scope
+		&& streq (sym->namesp->name, scope->name))))
       break;
 
   if (sym == NULL)
@@ -686,7 +684,7 @@ add_link (super, sub)
     {
       lnk = (struct link *) xmalloc (sizeof *lnk);
       lnk2 = (struct link *) xmalloc (sizeof *lnk2);
-  
+
       lnk->sym = sub;
       lnk->next = p;
 
@@ -728,15 +726,15 @@ find_member (cls, name, var, sc, hash)
     case SC_FRIEND:
       list = &cls->friends;
       break;
-      
+
     case SC_TYPE:
       list = &cls->types;
       break;
-      
+
     case SC_STATIC:
       list = var ? &cls->static_vars : &cls->static_fns;
       break;
-      
+
     default:
       list = var ? &cls->vars : &cls->fns;
       break;
@@ -928,7 +926,7 @@ add_global_defn (name, regexp, pos, hash, var, sc, flags)
    a bit set giving additional information about the member (see the
    F_* defines).  */
 
-void 
+void
 add_global_decl (name, regexp, pos, hash, var, sc, flags)
      char *name, *regexp;
      int pos;
@@ -1008,15 +1006,15 @@ add_member (cls, name, var, sc, hash)
     case SC_FRIEND:
       list = &cls->friends;
       break;
-      
+
     case SC_TYPE:
       list = &cls->types;
       break;
-      
+
     case SC_STATIC:
       list = var ? &cls->static_vars : &cls->static_fns;
       break;
-      
+
     default:
       list = var ? &cls->vars : &cls->fns;
       break;
@@ -1112,7 +1110,7 @@ check_namespace (name, context)
      struct sym *context;
 {
   struct sym *p = NULL;
-  
+
   for (p = all_namespaces; p; p = p->next)
     {
       if (streq (p->name, name) && (p->namesp == context))
@@ -1137,7 +1135,7 @@ find_namespace (name, context)
 
   return p;
 }
-  
+
 
 /* Find namespace alias with name NAME. If not found return NULL. */
 
@@ -1210,7 +1208,7 @@ enter_namespace (name)
 				    size * sizeof *namespace_stack);
       namespace_stack_size = size;
     }
-  
+
   namespace_stack[namespace_sp++] = current_namespace;
   current_namespace = p;
 }
@@ -1282,7 +1280,7 @@ sym_scope_1 (p)
      struct sym *p;
 {
   int len;
-  
+
   if (p->namesp)
     sym_scope_1 (p->namesp);
 
@@ -1297,14 +1295,14 @@ sym_scope_1 (p)
   ensure_scope_buffer_room (len + 1);
   strcat (scope_buffer, p->name);
   scope_buffer_len += len;
-  
+
   if (HAS_FLAG (p->flags, F_TEMPLATE))
     {
       ensure_scope_buffer_room (3);
       strcat (scope_buffer, "<>");
       scope_buffer_len += 2;
     }
-  
+
   return scope_buffer;
 }
 
@@ -1321,10 +1319,10 @@ sym_scope (p)
       scope_buffer_size = 1024;
       scope_buffer = (char *) xmalloc (scope_buffer_size);
     }
-  
+
   *scope_buffer = '\0';
   scope_buffer_len = 0;
-  
+
   if (p->namesp)
     sym_scope_1 (p->namesp);
 
@@ -1377,13 +1375,13 @@ dump_sym (fp, root)
 {
   fputs (CLASS_STRUCT, fp);
   PUTSTR (root->name, fp);
-  
+
   /* Print scope, if any.  */
   if (root->namesp)
     PUTSTR (sym_scope (root), fp);
   else
     PUTSTR (NULL, fp);
-  
+
   /* Print flags.  */
   fprintf (fp, "%u", root->flags);
   PUTSTR (root->filename, fp);
@@ -1563,7 +1561,7 @@ process_pp_line ()
 	  add_define (yytext, regexp, pos);
 	}
     }
-  
+
   while (c && (c != '\n' || in_comment || in_string))
     {
       if (c == '\\')
@@ -1580,7 +1578,7 @@ process_pp_line ()
 	}
       else if (c == '"')
 	in_string = !in_string;
-      
+
       if (c == '\n')
 	INCREMENT_LINENO;
 
@@ -2035,7 +2033,7 @@ matching_regexp ()
       while (in - p < min_regexp && p > inbuffer)
         {
           /* Line probably not significant enough */
-          for (--p; p >= inbuffer && *p != '\n'; --p)
+          for (--p; p > inbuffer && *p != '\n'; --p)
             ;
         }
       if (*p == '\n')
@@ -2184,7 +2182,7 @@ re_init_scanner ()
 {
   in = inbuffer;
   yyline = 1;
-  
+
   if (yytext == NULL)
     {
       int size = 256;
@@ -2386,19 +2384,19 @@ skip_matching ()
     case '{':
       close = '}';
       break;
-      
+
     case '(':
       close = ')';
       break;
-      
+
     case '<':
       close = '>';
       break;
-      
+
     case '[':
       close = ']';
       break;
-      
+
     default:
       abort ();
     }
@@ -2419,6 +2417,30 @@ skip_matching ()
     }
 }
 
+void
+skip_initializer ()
+{
+  for (;;)
+    {
+      switch (LA1)
+	{
+	case ';':
+	case ',':
+	case YYEOF:
+	  return;
+
+	case '{':
+	case '[':
+	case '(':
+	  skip_matching ();
+	  break;
+
+	default:
+	  MATCH ();
+	  break;
+	}
+    }
+}
 
 /* Build qualified namespace alias (A::B::c) and return it. */
 
@@ -2483,7 +2505,7 @@ parm_list (flags)
     {
       switch (LA1)
         {
-	  /* Skip over grouping parens or parameter lists in parameter 
+	  /* Skip over grouping parens or parameter lists in parameter
 	     declarations.  */
         case '(':
           skip_matching ();
@@ -2505,7 +2527,7 @@ parm_list (flags)
             {
 	      char *last_id;
 	      unsigned ident_type_hash = 0;
-	      
+
 	      parse_qualified_param_ident_or_type (&last_id);
 	      if (last_id)
 		{
@@ -2533,7 +2555,7 @@ parm_list (flags)
         case DOUBLE:    case ENUM:      case FLOAT:     case INT:
         case LONG:      case SHORT:     case SIGNED:    case STRUCT:
         case UNION:     case UNSIGNED:  case VOLATILE:  case WCHAR:
-        case ELLIPSIS:  
+        case ELLIPSIS:
           type_seen = 1;
           hash = (hash << 1) ^ LA1;
           MATCH ();
@@ -2553,7 +2575,7 @@ parm_list (flags)
   if (LOOKING_AT (')'))
     {
       MATCH ();
-      
+
       if (LOOKING_AT (CONST))
         {
           /* We can overload the same function on `const' */
@@ -2638,7 +2660,7 @@ member (cls, vis)
         case EXPLICIT:
           SET_FLAG (flags, F_EXPLICIT);
           goto typeseen;
-          
+
         case MUTABLE:
           SET_FLAG (flags, F_MUTABLE);
           goto typeseen;
@@ -2726,7 +2748,7 @@ member (cls, vis)
           if (LOOKING_AT ('{') && id && cls)
 	    add_member_defn (cls, id, regexp, pos, hash, 0, sc, flags);
 
-	  xfree (id);
+	  free (id);
           id = NULL;
           sc = SC_MEMBER;
           break;
@@ -2773,7 +2795,7 @@ member (cls, vis)
         {
           regexp = matching_regexp ();
           pos = BUFFER_POS ();
-          
+
           if (cls != NULL)
             {
               if (type_seen || !paren_seen)
@@ -2782,7 +2804,7 @@ member (cls, vis)
 		add_member_decl (cls, id, regexp, pos, hash, 0, sc, vis, 0);
             }
         }
-      
+
       MATCH ();
       print_info ();
     }
@@ -2805,7 +2827,7 @@ member (cls, vis)
       print_info ();
     }
 
-  xfree (id);
+  free (id);
 }
 
 
@@ -2879,7 +2901,7 @@ struct sym *
 parse_classname ()
 {
   struct sym *last_class = NULL;
-  
+
   while (LOOKING_AT (IDENT))
     {
       last_class = add_sym (yytext, last_class);
@@ -2890,10 +2912,10 @@ parse_classname ()
           skip_matching ();
           SET_FLAG (last_class->flags, F_TEMPLATE);
         }
-      
+
       if (!LOOKING_AT (DCOLON))
         break;
-      
+
       MATCH ();
     }
 
@@ -2913,7 +2935,7 @@ operator_name (sc)
   static char *id = NULL;
   char *s;
   int len;
-  
+
   MATCH ();
 
   if (LOOKING_AT2 (NEW, DELETE))
@@ -2924,7 +2946,7 @@ operator_name (sc)
 
       s = token_string (LA1);
       MATCH ();
-      
+
       len = strlen (s) + 10;
       if (len > id_size)
 	{
@@ -2934,12 +2956,12 @@ operator_name (sc)
 	}
       strcpy (id, s);
 
-      /* Vector new or delete? */
+      /* Vector new or delete?  */
       if (LOOKING_AT ('['))
 	{
 	  strcat (id, "[");
 	  MATCH ();
-	  
+
 	  if (LOOKING_AT (']'))
 	    {
 	      strcat (id, "]");
@@ -3004,7 +3026,7 @@ parse_qualified_ident_or_type (last_id)
   char *id = NULL;
   size_t id_size = 0;
   int enter = 0;
-  
+
   while (LOOKING_AT (IDENT))
     {
       int len = strlen (yytext) + 1;
@@ -3042,7 +3064,7 @@ parse_qualified_ident_or_type (last_id)
 	    cls = add_sym (id, cls);
 
 	  *last_id = NULL;
-	  xfree (id);
+	  free (id);
 	  id = NULL;
 	  id_size = 0;
 	  MATCH ();
@@ -3120,7 +3142,7 @@ class_definition (containing, tag, flags, nested)
      settings.  */
   if ((tag != CLASS && !f_structs) || (nested && !f_nested_classes))
     current = NULL;
-  else 
+  else
     {
       current = add_sym (yytext, containing);
       current->pos = BUFFER_POS ();
@@ -3139,7 +3161,7 @@ class_definition (containing, tag, flags, nested)
         {
           switch (LA1)
             {
-            case VIRTUAL: case PUBLIC: case PROTECTED: case PRIVATE: 
+            case VIRTUAL: case PUBLIC: case PROTECTED: case PRIVATE:
               MATCH ();
               break;
 
@@ -3201,6 +3223,54 @@ class_definition (containing, tag, flags, nested)
     }
 }
 
+/* Add to class *CLS information for the declaration of variable or
+   type *ID.  If *CLS is null, this means a global declaration.  SC is
+   the storage class of *ID.  FLAGS is a bit set giving additional
+   information about the member (see the F_* defines).  */
+
+void
+add_declarator (cls, id, flags, sc)
+     struct sym **cls;
+     char **id;
+     int flags, sc;
+{
+  if (LOOKING_AT2 (';', ','))
+    {
+      /* The end of a member variable or of an access declaration
+         `X::f'.  To distinguish between them we have to know whether
+         type information has been seen.  */
+      if (*id)
+        {
+          char *regexp = matching_regexp ();
+          int pos = BUFFER_POS ();
+
+          if (*cls)
+	    add_member_defn (*cls, *id, regexp, pos, 0, 1, SC_UNKNOWN, flags);
+          else
+            add_global_defn (*id, regexp, pos, 0, 1, sc, flags);
+        }
+
+      MATCH ();
+      print_info ();
+    }
+  else if (LOOKING_AT ('{'))
+    {
+      if (sc == SC_TYPE && *id)
+        {
+          /* A named enumeration.  */
+          char *regexp = matching_regexp ();
+          int pos = BUFFER_POS ();
+          add_global_defn (*id, regexp, pos, 0, 1, sc, flags);
+        }
+
+      skip_matching ();
+      print_info ();
+    }
+
+  free (*id);
+  *id = NULL;
+  *cls = NULL;
+}
 
 /* Parse a declaration.  */
 
@@ -3233,7 +3303,7 @@ declaration (flags)
           sc = SC_TYPE;
           MATCH ();
           break;
-          
+
         case STATIC:
           sc = SC_STATIC;
           MATCH ();
@@ -3250,16 +3320,20 @@ declaration (flags)
              `declare (X, Y)\n class A : ...'.  */
           if (id)
 	    {
-	      xfree (id);
+	      free (id);
 	      return;
 	    }
 
         case '=':
-          /* Assumed to be the start of an initialization in this context.
-             Skip over everything up to ';'.  */
-          skip_to (';');
+          /* Assumed to be the start of an initialization in this
+	     context.  */
+	  skip_initializer ();
           break;
 
+	case ',':
+	  add_declarator (&cls, &id, flags, sc);
+	  break;
+
         case OPERATOR:
 	  {
 	    char *s = operator_name (&sc);
@@ -3340,46 +3414,13 @@ declaration (flags)
           if (!cls && id && LOOKING_AT ('{'))
 	    add_global_defn (id, regexp, pos, hash, 0, sc, flags);
 
-	  xfree (id);
+	  free (id);
           id = NULL;
           break;
         }
     }
 
-  if (LOOKING_AT (';'))
-    {
-      /* The end of a member variable or of an access declaration
-         `X::f'.  To distinguish between them we have to know whether
-         type information has been seen.  */
-      if (id)
-        {
-          char *regexp = matching_regexp ();
-          int pos = BUFFER_POS ();
-
-          if (cls)
-	    add_member_defn (cls, id, regexp, pos, 0, 1, SC_UNKNOWN, flags);
-          else
-            add_global_defn (id, regexp, pos, 0, 1, sc, flags);
-        }
-        
-      MATCH ();
-      print_info ();
-    }
-  else if (LOOKING_AT ('{'))
-    {
-      if (sc == SC_TYPE && id)
-        {
-          /* A named enumeration.  */
-          regexp = matching_regexp ();
-          pos = BUFFER_POS ();
-          add_global_defn (id, regexp, pos, 0, 1, sc, flags);
-        }
-
-      skip_matching ();
-      print_info ();
-    }
-
-  xfree (id);
+  add_declarator (&cls, &id, flags, sc);
 }
 
 
@@ -3399,7 +3440,7 @@ globals (start_flags)
   for (;;)
     {
       char *prev_in = in;
-      
+
       switch (LA1)
         {
         case NAMESPACE:
@@ -3410,13 +3451,13 @@ globals (start_flags)
               {
                 char *namespace_name = xstrdup (yytext);
                 MATCH ();
-                
+
                 if (LOOKING_AT ('='))
                   {
 		    struct link *qna = match_qualified_namespace_alias ();
 		    if (qna)
                       register_namespace_alias (namespace_name, qna);
-		      
+
                     if (skip_to (';') == ';')
                       MATCH ();
                   }
@@ -3429,7 +3470,7 @@ globals (start_flags)
                     MATCH_IF ('}');
                   }
 
-		xfree (namespace_name);
+		free (namespace_name);
               }
           }
           break;
@@ -3441,7 +3482,7 @@ globals (start_flags)
             {
               /* This is `extern "C"'.  */
               MATCH ();
-              
+
               if (LOOKING_AT ('{'))
                 {
                   MATCH ();
@@ -3452,7 +3493,7 @@ globals (start_flags)
                 SET_FLAG (flags, F_EXTERNC);
             }
           break;
-          
+
         case TEMPLATE:
           MATCH ();
           SKIP_MATCHING_IF ('<');
@@ -3492,7 +3533,7 @@ globals (start_flags)
 
         case '}':
           return 0;
-          
+
         default:
           declaration (flags);
           flags = start_flags;
@@ -3531,10 +3572,10 @@ add_search_path (path_list)
     {
       char *start = path_list;
       struct search_path *p;
-      
+
       while (*path_list && *path_list != PATH_LIST_SEPARATOR)
         ++path_list;
-      
+
       p = (struct search_path *) xmalloc (sizeof *p);
       p->path = (char *) xmalloc (path_list - start + 1);
       memcpy (p->path, start, path_list - start);
@@ -3568,7 +3609,7 @@ open_file (file)
   static int buffer_size;
   struct search_path *path;
   int flen = strlen (file) + 1;	/* +1 for the slash */
-  
+
   filename = xstrdup (file);
 
   for (path = search_path; path && fp == NULL; path = path->next)
@@ -3580,20 +3621,20 @@ open_file (file)
 	  buffer_size = max (len + 1, 2 * buffer_size);
 	  buffer = (char *) xrealloc (buffer, buffer_size);
 	}
-	
+
       strcpy (buffer, path->path);
       strcat (buffer, "/");
       strcat (buffer, file);
       fp = fopen (buffer, "r");
     }
-  
+
   /* Try the original file name.  */
   if (fp == NULL)
      fp = fopen (file, "r");
 
   if (fp == NULL)
     yyerror ("cannot open", NULL);
-  
+
   return fp;
 }
 
@@ -3624,7 +3665,7 @@ usage (error)
      int error;
 {
   puts (USAGE);
-  exit (error ? 1 : 0);
+  exit (error ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
 
@@ -3638,10 +3679,13 @@ usage (error)
 void
 version ()
 {
+  /* Makes it easier to update automatically. */
+  char emacs_copyright[] = "Copyright (C) 2009 Free Software Foundation, Inc.";
+
   printf ("ebrowse %s\n", VERSION);
-  puts ("Copyright (C) 1992-1999, 2000, 2001 Free Software Foundation, Inc.");
+  puts (emacs_copyright);
   puts ("This program is distributed under the same terms as Emacs.");
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 
@@ -3653,10 +3697,10 @@ process_file (file)
      char *file;
 {
   FILE *fp;
-  
+
   fp = open_file (file);
   if (fp)
-    {      
+    {
       int nread, nbytes;
 
       /* Give a progress indication if needed.  */
@@ -3679,7 +3723,7 @@ process_file (file)
 	      inbuffer_size = nread + READ_CHUNK_SIZE + 1;
 	      inbuffer = (char *) xrealloc (inbuffer, inbuffer_size);
 	    }
-	  
+
 	  nbytes = fread (inbuffer + nread, 1, READ_CHUNK_SIZE, fp);
 	  if (nbytes <= 0)
 	    break;
@@ -3722,10 +3766,10 @@ read_line (fp)
 
       buffer[i++] = c;
     }
-  
+
   if (c == EOF && i == 0)
     return NULL;
-  
+
   if (i == buffer_size)
     {
       buffer_size = max (100, buffer_size * 2);
@@ -3765,7 +3809,7 @@ main (argc, argv)
 	case 'p':
 	  info_position = atoi (optarg);
 	  break;
-	  
+
         case 'n':
           f_nested_classes = 0;
           break;
@@ -3773,7 +3817,7 @@ main (argc, argv)
         case 'x':
           f_regexps = 0;
           break;
-        
+
           /* Add the name of a file containing more input files.  */
         case 'f':
 	  if (n_input_files == input_filenames_size)
@@ -3859,26 +3903,40 @@ main (argc, argv)
 
 	  fp = fopen (out_filename, "r");
 	  if (fp == NULL)
-	    yyerror ("file `%s' must exist for --append", out_filename);
+	    {
+	      yyerror ("file `%s' must exist for --append", out_filename);
+	      exit (EXIT_FAILURE);
+	    }
 
 	  rc = fseek (fp, 0, SEEK_END);
 	  if (rc == -1)
-	    yyerror ("error seeking in file `%s'", out_filename);
+	    {
+	      yyerror ("error seeking in file `%s'", out_filename);
+	      exit (EXIT_FAILURE);
+	    }
 
 	  rc = ftell (fp);
 	  if (rc == -1)
-	    yyerror ("error getting size of file `%s'", out_filename);
-	  else if (rc == 0)
-	    yyerror ("file `%s' is empty", out_filename);
+	    {
+	      yyerror ("error getting size of file `%s'", out_filename);
+	      exit (EXIT_FAILURE);
+	    }
 	  
+	  else if (rc == 0)
+	    {
+	      yyerror ("file `%s' is empty", out_filename);
+	      /* It may be ok to use an empty file for appending.
+		 exit (EXIT_FAILURE); */
+	    }
+
 	  fclose (fp);
 	}
-      
+
       yyout = fopen (out_filename, f_append ? "a" : "w");
       if (yyout == NULL)
 	{
 	  yyerror ("cannot open output file `%s'", out_filename);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 
@@ -3903,7 +3961,7 @@ main (argc, argv)
       for (i = 0; i < n_input_files; ++i)
         {
           FILE *fp = fopen (input_filenames[i], "r");
-          
+
           if (fp == NULL)
             yyerror ("cannot open input file `%s'", input_filenames[i]);
           else
@@ -3923,8 +3981,10 @@ main (argc, argv)
   if (yyout != stdout)
     fclose (yyout);
 
-  return 0;
+  return EXIT_SUCCESS;
 }
 
+/* arch-tag: fc03b4bc-91a9-4c3d-b3b9-12a77fa86dd8
+   (do not change this comment) */
 
-/* ebrowse.c ends here.  */
+/* ebrowse.c ends here */
