@@ -1,12 +1,13 @@
 /* Terminal hooks for GNU Emacs on the Microsoft W32 API.
-   Copyright (C) 1992, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1992, 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+                 2008, 2009  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,9 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.
+along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
+/*
    Tim Fleehart (apollo@online.com)		1-17-92
    Geoff Voelker (voelker@cs.washington.edu)	9-12-93
 */
@@ -31,18 +31,15 @@ Boston, MA 02111-1307, USA.
 #include <string.h>
 
 #include "lisp.h"
-#include "charset.h"
+#include "character.h"
 #include "coding.h"
 #include "disptab.h"
-#include "termhooks.h"
-#include "dispextern.h"
-/* Disable features in frame.h that require a Window System.  */
-#undef HAVE_WINDOW_SYSTEM
 #include "frame.h"
+#include "termhooks.h"
+#include "termchar.h"
+#include "dispextern.h"
 #include "w32inevt.h"
 
-#define min(a, b) ((a) < (b) ? (a) : (b))
-
 /* from window.c */
 extern Lisp_Object Frecenter ();
 
@@ -52,27 +49,20 @@ extern int detect_input_pending ();
 /* from sysdep.c */
 extern int read_input_pending ();
 
-extern struct frame * updating_frame;
-extern int meta_key;
-
-static void move_cursor (int row, int col);
-static void clear_to_end (void);
-static void clear_frame (void);
-static void clear_end_of_line (int);
-static void ins_del_lines (int vpos, int n);
-static void change_line_highlight (int, int, int, int);
-static void reassert_line_highlight (int, int);
-static void insert_glyphs (struct glyph *start, int len);
-static void write_glyphs (struct glyph *string, int len);
-static void delete_glyphs (int n);
-void w32_sys_ring_bell (void);
-static void reset_terminal_modes (void);
-static void set_terminal_modes (void);
-static void set_terminal_window (int size);
-static void update_begin (struct frame * f);
-static void update_end (struct frame * f);
+static void w32con_move_cursor (struct frame *f, int row, int col);
+static void w32con_clear_to_end (struct frame *f);
+static void w32con_clear_frame (struct frame *f);
+static void w32con_clear_end_of_line (struct frame *f, int);
+static void w32con_ins_del_lines (struct frame *f, int vpos, int n);
+static void w32con_insert_glyphs (struct frame *f, struct glyph *start, int len);
+static void w32con_write_glyphs (struct frame *f, struct glyph *string, int len);
+static void w32con_delete_glyphs (struct frame *f, int n);
+static void w32con_reset_terminal_modes (struct terminal *t);
+static void w32con_set_terminal_modes (struct terminal *t);
+static void w32con_set_terminal_window (struct frame *f, int size);
+static void w32con_update_begin (struct frame * f);
+static void w32con_update_end (struct frame * f);
 static WORD w32_face_attributes (struct frame *f, int face_id);
-static int hl_mode (int new_highlight);
 
 static COORD	cursor_coords;
 static HANDLE	prev_screen, cur_screen;
@@ -83,6 +73,8 @@ static DWORD   prev_console_mode;
 static CONSOLE_CURSOR_INFO prev_console_cursor;
 #endif
 
+extern Lisp_Object Vtty_defined_color_alist;
+
 /* Determine whether to make frame dimensions match the screen buffer,
    or the current window size.  The former is desirable when running
    over telnet, while the latter is more useful when working directly at
@@ -103,38 +95,31 @@ ctrl_c_handler (unsigned long type)
 	  && (type == CTRL_C_EVENT || type == CTRL_BREAK_EVENT));
 }
 
-/* If we're updating a frame, use it as the current frame
-   Otherwise, use the selected frame.  */
-#define PICK_FRAME() (updating_frame ? updating_frame : SELECTED_FRAME ())
 
-/* Move the cursor to (row, col).  */
-void
-move_cursor (int row, int col)
+/* Move the cursor to (ROW, COL) on FRAME.  */
+static void
+w32con_move_cursor (struct frame *f, int row, int col)
 {
   cursor_coords.X = col;
   cursor_coords.Y = row;
-  
-  if (updating_frame == (struct frame *) NULL)
-    {
-      SetConsoleCursorPosition (cur_screen, cursor_coords);
-    }
+
+  /* TODO: for multi-tty support, cur_screen should be replaced with a
+     reference to the terminal for this frame.  */
+  SetConsoleCursorPosition (cur_screen, cursor_coords);
 }
 
 /* Clear from cursor to end of screen.  */
-void
-clear_to_end (void)
+static void
+w32con_clear_to_end (struct frame *f)
 {
-  struct frame * f = PICK_FRAME ();
-  
-  clear_end_of_line (FRAME_WIDTH (f) - 1);
-  ins_del_lines (cursor_coords.Y, FRAME_HEIGHT (f) - cursor_coords.Y - 1);
+  w32con_clear_end_of_line (f, FRAME_COLS (f) - 1);
+  w32con_ins_del_lines (f, cursor_coords.Y, FRAME_LINES (f) - cursor_coords.Y - 1);
 }
 
 /* Clear the frame.  */
-void
-clear_frame (void)
+static void
+w32con_clear_frame (struct frame *f)
 {
-  struct frame *  f = PICK_FRAME ();
   COORD	     dest;
   int        n;
   DWORD      r;
@@ -142,16 +127,14 @@ clear_frame (void)
 
   GetConsoleScreenBufferInfo (GetStdHandle (STD_OUTPUT_HANDLE), &info);
 
-  hl_mode (0);
-
   /* Remember that the screen buffer might be wider than the window.  */
-  n = FRAME_HEIGHT (f) * info.dwSize.X;
+  n = FRAME_LINES (f) * info.dwSize.X;
   dest.X = dest.Y = 0;
 
   FillConsoleOutputAttribute (cur_screen, char_attr_normal, n, dest, &r);
   FillConsoleOutputCharacter (cur_screen, ' ', n, dest, &r);
 
-  move_cursor (0, 0);
+  w32con_move_cursor (f, 0, 0);
 }
 
 
@@ -159,8 +142,8 @@ static struct glyph glyph_base[256];
 static BOOL  ceol_initialized = FALSE;
 
 /* Clear from Cursor to end (what's "standout marker"?).  */
-void
-clear_end_of_line (int end)
+static void
+w32con_clear_end_of_line (struct frame *f, int end)
 {
   if (!ceol_initialized)
     {
@@ -171,42 +154,41 @@ clear_end_of_line (int end)
         }
       ceol_initialized = TRUE;
     }
-  write_glyphs (glyph_base, end - cursor_coords.X);	/* fencepost ?	*/
+  w32con_write_glyphs (f, glyph_base, end - cursor_coords.X);	/* fencepost ?	*/
 }
 
 /* Insert n lines at vpos. if n is negative delete -n lines.  */
-void
-ins_del_lines (int vpos, int n)
+static void
+w32con_ins_del_lines (struct frame *f, int vpos, int n)
 {
-  int	     i, nb, save_highlight;
+  int	     i, nb;
   SMALL_RECT scroll;
+  SMALL_RECT clip;
   COORD	     dest;
   CHAR_INFO  fill;
-  struct frame *  f = PICK_FRAME ();
 
   if (n < 0)
     {
       scroll.Top = vpos - n;
-      scroll.Bottom = FRAME_HEIGHT (f);
+      scroll.Bottom = FRAME_LINES (f);
       dest.Y = vpos;
     }
   else
     {
       scroll.Top = vpos;
-      scroll.Bottom = FRAME_HEIGHT (f) - n;
+      scroll.Bottom = FRAME_LINES (f) - n;
       dest.Y = vpos + n;
     }
-  scroll.Left = 0;
-  scroll.Right = FRAME_WIDTH (f);
-  
+  clip.Top = clip.Left = scroll.Left = 0;
+  clip.Right = scroll.Right = FRAME_COLS (f);
+  clip.Bottom = FRAME_LINES (f);
+
   dest.X = 0;
-  
-  save_highlight = hl_mode (0);
-  
+
   fill.Char.AsciiChar = 0x20;
   fill.Attributes = char_attr_normal;
-  
-  ScrollConsoleScreenBuffer (cur_screen, &scroll, NULL, dest, &fill);
+
+  ScrollConsoleScreenBuffer (cur_screen, &scroll, &clip, dest, &fill);
 
   /* Here we have to deal with a w32 console flake: If the scroll
      region looks like abc and we scroll c to a and fill with d we get
@@ -221,8 +203,8 @@ ins_del_lines (int vpos, int n)
         {
 	  for (i = scroll.Bottom; i < dest.Y; i++)
             {
-	      move_cursor (i, 0);
-	      clear_end_of_line (FRAME_WIDTH (f));
+	      w32con_move_cursor (f, i, 0);
+	      w32con_clear_end_of_line (f, FRAME_COLS (f));
             }
         }
     }
@@ -231,52 +213,17 @@ ins_del_lines (int vpos, int n)
       nb = dest.Y + (scroll.Bottom - scroll.Top) + 1;
 
       if (nb < scroll.Top)
-        { 
+        {
 	  for (i = nb; i < scroll.Top; i++)
             {
-	      move_cursor (i, 0);
-	      clear_end_of_line (FRAME_WIDTH (f));
+	      w32con_move_cursor (f, i, 0);
+	      w32con_clear_end_of_line (f, FRAME_COLS (f));
             }
         }
     }
-  
+
   cursor_coords.X = 0;
   cursor_coords.Y = vpos;
-  
-  hl_mode (save_highlight);
-}
-
-/* Changes attribute to use when drawing characters to control.  */
-static int
-hl_mode (int new_highlight)
-{
-  static int highlight = 0;
-  int old_highlight;
-  
-  old_highlight = highlight;
-  highlight = (new_highlight != 0);
-
-  return old_highlight;
-}
-
-/* Call this when about to modify line at position VPOS and change whether it
-   is highlighted.  */
-void
-change_line_highlight (int new_highlight, int vpos, int y, 
-                       int first_unused_hpos)
-{
-  hl_mode (new_highlight);
-  move_cursor (vpos, 0);
-  clear_end_of_line (first_unused_hpos);
-}
-
-/* External interface to control of standout mode. Call this when about to
- * modify line at position VPOS and not change whether it is highlighted.  */
-void
-reassert_line_highlight (int highlight, int vpos)
-{
-  hl_mode (highlight);
-  vpos;				/* pedantic compiler silencer */
 }
 
 #undef	LEFT
@@ -284,83 +231,90 @@ reassert_line_highlight (int highlight, int vpos)
 #define	LEFT	1
 #define	RIGHT	0
 
-void
-scroll_line (int dist, int direction)
+static void
+scroll_line (struct frame *f, int dist, int direction)
 {
   /* The idea here is to implement a horizontal scroll in one line to
      implement delete and half of insert.  */
-  SMALL_RECT scroll;
+  SMALL_RECT scroll, clip;
   COORD	     dest;
   CHAR_INFO  fill;
-  struct frame *  f = PICK_FRAME ();
-  
-  scroll.Top = cursor_coords.Y;
-  scroll.Bottom = cursor_coords.Y;
-  
+
+  clip.Top = scroll.Top = clip.Bottom = scroll.Bottom = cursor_coords.Y;
+  clip.Left = 0;
+  clip.Right = FRAME_COLS (f);
+
   if (direction == LEFT)
     {
       scroll.Left = cursor_coords.X + dist;
-      scroll.Right = FRAME_WIDTH (f) - 1;
+      scroll.Right = FRAME_COLS (f) - 1;
     }
   else
     {
       scroll.Left = cursor_coords.X;
-      scroll.Right = FRAME_WIDTH (f) - dist - 1;
+      scroll.Right = FRAME_COLS (f) - dist - 1;
     }
-  
+
   dest.X = cursor_coords.X;
   dest.Y = cursor_coords.Y;
-  
+
   fill.Char.AsciiChar = 0x20;
   fill.Attributes = char_attr_normal;
 
-  ScrollConsoleScreenBuffer (cur_screen, &scroll, NULL, dest, &fill);
+  ScrollConsoleScreenBuffer (cur_screen, &scroll, &clip, dest, &fill);
 }
 
 
 /* If start is zero insert blanks instead of a string at start ?. */
-void
-insert_glyphs (register struct glyph *start, register int len)
+static void
+w32con_insert_glyphs (struct frame *f, register struct glyph *start, register int len)
 {
-  scroll_line (len, RIGHT);
+  scroll_line (f, len, RIGHT);
 
   /* Move len chars to the right starting at cursor_coords, fill with blanks */
   if (start)
     {
       /* Print the first len characters of start, cursor_coords.X adjusted
 	 by write_glyphs.  */
-	
-      write_glyphs (start, len);
+
+      w32con_write_glyphs (f, start, len);
     }
   else
     {
-      clear_end_of_line (cursor_coords.X + len);
+      w32con_clear_end_of_line (f, cursor_coords.X + len);
     }
 }
 
-void
-write_glyphs (register struct glyph *string, register int len)
+extern unsigned char *encode_terminal_code P_ ((struct glyph *, int, 
+						struct coding_system *));
+
+static void
+w32con_write_glyphs (struct frame *f, register struct glyph *string,
+                     register int len)
 {
-  int produced, consumed;
   DWORD r;
-  struct frame * f = PICK_FRAME ();
   WORD char_attr;
-  unsigned char conversion_buffer[1024];
-  int conversion_buffer_size = sizeof conversion_buffer;
+  unsigned char *conversion_buffer;
+  struct coding_system *coding;
 
   if (len <= 0)
     return;
 
+  /* If terminal_coding does any conversion, use it, otherwise use
+     safe_terminal_coding.  We can't use CODING_REQUIRE_ENCODING here
+     because it always return 1 if the member src_multibyte is 1.  */
+  coding = (FRAME_TERMINAL_CODING (f)->common_flags & CODING_REQUIRE_ENCODING_MASK
+	    ? FRAME_TERMINAL_CODING (f) : &safe_terminal_coding);
   /* The mode bit CODING_MODE_LAST_BLOCK should be set to 1 only at
      the tail.  */
-  terminal_coding.mode &= ~CODING_MODE_LAST_BLOCK;
+  coding->mode &= ~CODING_MODE_LAST_BLOCK;
 
   while (len > 0)
     {
       /* Identify a run of glyphs with the same face.  */
       int face_id = string->face_id;
       int n;
-      
+
       for (n = 1; n < len; ++n)
 	if (string[n].face_id != face_id)
 	  break;
@@ -368,90 +322,58 @@ write_glyphs (register struct glyph *string, register int len)
       /* Turn appearance modes of the face of the run on.  */
       char_attr = w32_face_attributes (f, face_id);
 
-      while (n > 0)
-        {
-	  /* We use a fixed size (1024 bytes) of conversion buffer.
-	     Usually it is sufficient, but if not, we just repeat the
-	     loop.  */
-	  produced = encode_terminal_code (string, conversion_buffer,
-					   n, conversion_buffer_size,
-					   &consumed);
-	  if (produced > 0)
+      if (n == len)
+	/* This is the last run.  */
+	coding->mode |= CODING_MODE_LAST_BLOCK;
+      conversion_buffer = encode_terminal_code (string, n, coding);
+      if (coding->produced > 0)
+	{
+	  /* Set the attribute for these characters.  */
+	  if (!FillConsoleOutputAttribute (cur_screen, char_attr,
+					   coding->produced, cursor_coords,
+					   &r))
 	    {
-              /* Set the attribute for these characters.  */
-              if (!FillConsoleOutputAttribute (cur_screen, char_attr,
-                                               produced, cursor_coords, &r)) 
-                {
-                  printf ("Failed writing console attributes: %d\n",
-                          GetLastError ());
-                  fflush (stdout);
-                }
+	      printf ("Failed writing console attributes: %d\n",
+		      GetLastError ());
+	      fflush (stdout);
+	    }
 
-              /* Write the characters.  */
-              if (!WriteConsoleOutputCharacter (cur_screen, conversion_buffer,
-                                                produced, cursor_coords, &r))
-                {
-                  printf ("Failed writing console characters: %d\n",
-                          GetLastError ());
-                  fflush (stdout);
-                }
+	  /* Write the characters.  */
+	  if (!WriteConsoleOutputCharacter (cur_screen, conversion_buffer,
+					    coding->produced, cursor_coords,
+					    &r))
+	    {
+	      printf ("Failed writing console characters: %d\n",
+		      GetLastError ());
+	      fflush (stdout);
+	    }
 
-              cursor_coords.X += produced;
-              move_cursor (cursor_coords.Y, cursor_coords.X);
-            }    
-          len -= consumed;
-          n -= consumed;
-          string += consumed;
-        }
-    }
-
-  /* We may have to output some codes to terminate the writing.  */
-  if (CODING_REQUIRE_FLUSHING (&terminal_coding))
-    {
-      terminal_coding.mode |= CODING_MODE_LAST_BLOCK;
-      encode_coding (&terminal_coding, "", conversion_buffer,
-		     0, conversion_buffer_size);
-      if (terminal_coding.produced > 0)
-        {
-          if (!FillConsoleOutputAttribute (cur_screen, char_attr_normal,
-                                           terminal_coding.produced,
-                                           cursor_coords, &r)) 
-            {
-              printf ("Failed writing console attributes: %d\n",
-                      GetLastError ());
-              fflush (stdout);
-            }
-
-          /* Write the characters.  */
-          if (!WriteConsoleOutputCharacter (cur_screen, conversion_buffer,
-                                            produced, cursor_coords, &r))
-            {
-              printf ("Failed writing console characters: %d\n",
-                      GetLastError ());
-              fflush (stdout);
-            }
-        }
+	  cursor_coords.X += coding->produced;
+	  w32con_move_cursor (f, cursor_coords.Y, cursor_coords.X);
+	}
+      len -= n;
+      string += n;
     }
 }
 
 
-void
-delete_glyphs (int n)
+static void
+w32con_delete_glyphs (struct frame *f, int n)
 {
-  /* delete chars means scroll chars from cursor_coords.X + n to 
-     cursor_coords.X, anything beyond the edge of the screen should 
+  /* delete chars means scroll chars from cursor_coords.X + n to
+     cursor_coords.X, anything beyond the edge of the screen should
      come out empty...  */
 
-  scroll_line (n, LEFT);
+  scroll_line (f, n, LEFT);
 }
 
 static unsigned int sound_type = 0xFFFFFFFF;
 #define MB_EMACS_SILENT (0xFFFFFFFF - 1)
 
 void
-w32_sys_ring_bell (void)
+w32_sys_ring_bell (struct frame *f)
 {
-  if (sound_type == 0xFFFFFFFF) 
+  if (sound_type == 0xFFFFFFFF)
     {
       Beep (666, 100);
     }
@@ -464,27 +386,27 @@ w32_sys_ring_bell (void)
 }
 
 DEFUN ("set-message-beep", Fset_message_beep, Sset_message_beep, 1, 1, 0,
-       "Set the sound generated when the bell is rung.\n\
-SOUND is 'asterisk, 'exclamation, 'hand, 'question, 'ok, or 'silent\n\
-to use the corresponding system sound for the bell.  The 'silent sound\n\
-prevents Emacs from making any sound at all.\n\
-SOUND is nil to use the normal beep.")
+       doc: /* Set the sound generated when the bell is rung.
+SOUND is 'asterisk, 'exclamation, 'hand, 'question, 'ok, or 'silent
+to use the corresponding system sound for the bell.  The 'silent sound
+prevents Emacs from making any sound at all.
+SOUND is nil to use the normal beep.  */)
      (sound)
      Lisp_Object sound;
 {
-  CHECK_SYMBOL (sound, 0);
+  CHECK_SYMBOL (sound);
 
-  if (NILP (sound)) 
+  if (NILP (sound))
       sound_type = 0xFFFFFFFF;
   else if (EQ (sound, intern ("asterisk")))
       sound_type = MB_ICONASTERISK;
-  else if (EQ (sound, intern ("exclamation"))) 
+  else if (EQ (sound, intern ("exclamation")))
       sound_type = MB_ICONEXCLAMATION;
-  else if (EQ (sound, intern ("hand"))) 
+  else if (EQ (sound, intern ("hand")))
       sound_type = MB_ICONHAND;
-  else if (EQ (sound, intern ("question"))) 
+  else if (EQ (sound, intern ("question")))
       sound_type = MB_ICONQUESTION;
-  else if (EQ (sound, intern ("ok"))) 
+  else if (EQ (sound, intern ("ok")))
       sound_type = MB_OK;
   else if (EQ (sound, intern ("silent")))
       sound_type = MB_EMACS_SILENT;
@@ -493,27 +415,43 @@ SOUND is nil to use the normal beep.")
 
   return sound;
 }
-   
-void
-reset_terminal_modes (void)
-{
-  hl_mode (0);
 
+static void
+w32con_reset_terminal_modes (struct terminal *t)
+{
+  COORD dest;
+  CONSOLE_SCREEN_BUFFER_INFO info;
+  int n;
+  DWORD r;
+
+  /* Clear the complete screen buffer.  This is required because Emacs
+     sets the cursor position to the top of the buffer, but there might
+     be other output below the bottom of the Emacs frame if the screen buffer
+     is larger than the window size.  */
+  GetConsoleScreenBufferInfo (cur_screen, &info);
+  dest.X = 0;
+  dest.Y = 0;
+  n = info.dwSize.X * info.dwSize.Y;
+
+  FillConsoleOutputAttribute (cur_screen, char_attr_normal, n, dest, &r);
+  FillConsoleOutputCharacter (cur_screen, ' ', n, dest, &r);
+  /* Now that the screen is clear, put the cursor at the top.  */
+  SetConsoleCursorPosition (cur_screen, dest);
+  
 #ifdef USE_SEPARATE_SCREEN
   SetConsoleActiveScreenBuffer (prev_screen);
 #else
   SetConsoleCursorInfo (prev_screen, &prev_console_cursor);
 #endif
+
   SetConsoleMode (keyboard_handle, prev_console_mode);
 }
 
-void
-set_terminal_modes (void)
+static void
+w32con_set_terminal_modes (struct terminal *t)
 {
   CONSOLE_CURSOR_INFO cci;
 
-  hl_mode (0);
-
   /* make cursor big and visible (100 on Win95 makes it disappear)  */
   cci.dwSize = 99;
   cci.bVisible = TRUE;
@@ -530,23 +468,21 @@ set_terminal_modes (void)
 
 /* hmmm... perhaps these let us bracket screen changes so that we can flush
    clumps rather than one-character-at-a-time...
-   
+
    we'll start with not moving the cursor while an update is in progress.  */
-void
-update_begin (struct frame * f)
+static void
+w32con_update_begin (struct frame * f)
 {
-  hl_mode (0);
 }
 
-void
-update_end (struct frame * f)
+static void
+w32con_update_end (struct frame * f)
 {
-  hl_mode (0);
   SetConsoleCursorPosition (cur_screen, cursor_coords);
 }
 
-void
-set_terminal_window (int size)
+static void
+w32con_set_terminal_window (struct frame *f, int size)
 {
 }
 
@@ -563,43 +499,38 @@ w32_face_attributes (f, face_id)
      int face_id;
 {
   WORD char_attr;
-  int highlight_on_p;
   struct face *face = FACE_FROM_ID (f, face_id);
 
-  highlight_on_p = hl_mode (0);
-  hl_mode (highlight_on_p);
-
   xassert (face != NULL);
 
   char_attr = char_attr_normal;
 
-  if (face->foreground != FACE_TTY_DEFAULT_FG_COLOR
-      && face->foreground != FACE_TTY_DEFAULT_COLOR)
-    char_attr = (char_attr & 0xfff0) + (face->foreground % 16);
-
-  if (face->background != FACE_TTY_DEFAULT_BG_COLOR
-      && face->background != FACE_TTY_DEFAULT_COLOR)
-    char_attr = (char_attr & 0xff0f) + ((face->background % 16) << 4); 
-
-
-  /* NTEMACS_TODO: Faces defined during startup get both foreground
-     and background of 0. Need a better way around this - for now detect
-     the problem and invert one of the faces to make the text readable. */
-  if (((char_attr & 0x00f0) >> 4) == (char_attr & 0x000f))
-    char_attr ^= 0x0007;
-
-  if (face->tty_reverse_p || highlight_on_p)
+  /* Reverse the default color if requested. If background and
+     foreground are specified, then they have been reversed already.  */
+  if (face->tty_reverse_p)
     char_attr = (char_attr & 0xff00) + ((char_attr & 0x000f) << 4)
       + ((char_attr & 0x00f0) >> 4);
 
+  /* Before the terminal is properly initialized, all colors map to 0.
+     Don't try to resolve them.  */
+  if (NILP (Vtty_defined_color_alist))
+    return char_attr;
+
+  /* Colors should be in the range 0...15 unless they are one of
+     FACE_TTY_DEFAULT_COLOR, FACE_TTY_DEFAULT_FG_COLOR or
+     FACE_TTY_DEFAULT_BG_COLOR.  Other out of range colors are
+     invalid, so it is better to use the default color if they ever
+     get through to here.  */
+  if (face->foreground >= 0 && face->foreground < 16)
+    char_attr = (char_attr & 0xfff0) + face->foreground;
+
+  if (face->background >= 0 && face->background < 16)
+    char_attr = (char_attr & 0xff0f) + (face->background << 4);
+
   return char_attr;
 }
 
 
-/* Emulation of some X window features from xfns.c and xfaces.c.  */
-
-extern char unspecified_fg[], unspecified_bg[];
-
 
 /* Given a color index, return its standard name.  */
 Lisp_Object
@@ -624,31 +555,38 @@ vga_stdcolor_name (int idx)
 typedef int (*term_hook) ();
 
 void
-initialize_w32_display (void)
+initialize_w32_display (struct terminal *term)
 {
   CONSOLE_SCREEN_BUFFER_INFO	info;
-  
-  cursor_to_hook		= move_cursor;
-  raw_cursor_to_hook		= move_cursor;
-  clear_to_end_hook		= clear_to_end;
-  clear_frame_hook		= clear_frame;
-  clear_end_of_line_hook	= clear_end_of_line;
-  ins_del_lines_hook		= ins_del_lines;
-  change_line_highlight_hook	= change_line_highlight;
-  reassert_line_highlight_hook  = reassert_line_highlight;
-  insert_glyphs_hook		= insert_glyphs;
-  write_glyphs_hook		= write_glyphs;
-  delete_glyphs_hook		= delete_glyphs;
-  ring_bell_hook		= w32_sys_ring_bell;
-  reset_terminal_modes_hook	= reset_terminal_modes;
-  set_terminal_modes_hook	= set_terminal_modes;
-  set_terminal_window_hook	= set_terminal_window;
-  update_begin_hook		= update_begin;
-  update_end_hook		= update_end;
-  
-  read_socket_hook = w32_console_read_socket;
-  mouse_position_hook = w32_console_mouse_position;
-  estimate_mode_line_height_hook = 0;
+
+  term->rif = 0; /* No window based redisplay on the console.  */
+  term->cursor_to_hook		= w32con_move_cursor;
+  term->raw_cursor_to_hook		= w32con_move_cursor;
+  term->clear_to_end_hook		= w32con_clear_to_end;
+  term->clear_frame_hook		= w32con_clear_frame;
+  term->clear_end_of_line_hook	= w32con_clear_end_of_line;
+  term->ins_del_lines_hook		= w32con_ins_del_lines;
+  term->insert_glyphs_hook		= w32con_insert_glyphs;
+  term->write_glyphs_hook		= w32con_write_glyphs;
+  term->delete_glyphs_hook		= w32con_delete_glyphs;
+  term->ring_bell_hook		= w32_sys_ring_bell;
+  term->reset_terminal_modes_hook	= w32con_reset_terminal_modes;
+  term->set_terminal_modes_hook	= w32con_set_terminal_modes;
+  term->set_terminal_window_hook	= w32con_set_terminal_window;
+  term->update_begin_hook		= w32con_update_begin;
+  term->update_end_hook		= w32con_update_end;
+
+  term->read_socket_hook = w32_console_read_socket;
+  term->mouse_position_hook = w32_console_mouse_position;
+
+  /* The following are not used on the console.  */
+  term->frame_rehighlight_hook = 0;
+  term->frame_raise_lower_hook = 0;
+  term->set_vertical_scroll_bar_hook = 0;
+  term->condemn_scroll_bars_hook = 0;
+  term->redeem_scroll_bar_hook = 0;
+  term->judge_scroll_bars_hook = 0;
+  term->frame_up_to_date_hook = 0;
 
   /* Initialize interrupt_handle.  */
   init_crit ();
@@ -658,7 +596,7 @@ initialize_w32_display (void)
   GetConsoleMode (keyboard_handle, &prev_console_mode);
 
   prev_screen = GetStdHandle (STD_OUTPUT_HANDLE);
-  
+
 #ifdef USE_SEPARATE_SCREEN
   cur_screen = CreateConsoleScreenBuffer (GENERIC_READ | GENERIC_WRITE,
 					  0, NULL,
@@ -712,23 +650,38 @@ initialize_w32_display (void)
   }
 
   GetConsoleScreenBufferInfo (cur_screen, &info);
-  
-  meta_key = 1;
-  char_attr_normal = info.wAttributes;
-  hl_mode (0);
 
-  if (w32_use_full_screen_buffer)
+  char_attr_normal = info.wAttributes;
+
+  /* Determine if the info returned by GetConsoleScreenBufferInfo
+     is realistic.  Old MS Telnet servers used to only fill out
+     the dwSize portion, even modern one fill the whole struct with
+     garbage when using non-MS telnet clients.  */
+  if ((w32_use_full_screen_buffer
+       && (info.dwSize.Y < 20 || info.dwSize.Y > 100
+	   || info.dwSize.X < 40 || info.dwSize.X > 200))
+      || (!w32_use_full_screen_buffer
+	  && (info.srWindow.Bottom - info.srWindow.Top < 20
+	      || info.srWindow.Bottom - info.srWindow.Top > 100
+	      || info.srWindow.Right - info.srWindow.Left < 40
+	      || info.srWindow.Right - info.srWindow.Left > 100)))
     {
-      FRAME_HEIGHT (SELECTED_FRAME ()) = info.dwSize.Y;	/* lines per page */
-      SET_FRAME_WIDTH (SELECTED_FRAME (), info.dwSize.X);  /* characters per line */
+      FRAME_LINES (SELECTED_FRAME ()) = 25;
+      SET_FRAME_COLS (SELECTED_FRAME (), 80);
+    }
+
+  else if (w32_use_full_screen_buffer)
+    {
+      FRAME_LINES (SELECTED_FRAME ()) = info.dwSize.Y;	/* lines per page */
+      SET_FRAME_COLS (SELECTED_FRAME (), info.dwSize.X);  /* characters per line */
     }
   else
     {
       /* Lines per page.  Use buffer coords instead of buffer size.  */
-      FRAME_HEIGHT (SELECTED_FRAME ()) = 1 + info.srWindow.Bottom - 
-	info.srWindow.Top; 
+      FRAME_LINES (SELECTED_FRAME ()) = 1 + info.srWindow.Bottom -
+	info.srWindow.Top;
       /* Characters per line.  Use buffer coords instead of buffer size.  */
-      SET_FRAME_WIDTH (SELECTED_FRAME (), 1 + info.srWindow.Right - 
+      SET_FRAME_COLS (SELECTED_FRAME (), 1 + info.srWindow.Right -
 		       info.srWindow.Left);
     }
 
@@ -738,8 +691,9 @@ initialize_w32_display (void)
 
 }
 
+
 DEFUN ("set-screen-color", Fset_screen_color, Sset_screen_color, 2, 2, 0,
-       "Set screen colors.")
+       doc: /* Set screen colors.  */)
     (foreground, background)
     Lisp_Object foreground;
     Lisp_Object background;
@@ -751,7 +705,7 @@ DEFUN ("set-screen-color", Fset_screen_color, Sset_screen_color, 2, 2, 0,
 }
 
 DEFUN ("set-cursor-size", Fset_cursor_size, Sset_cursor_size, 1, 1, 0,
-       "Set cursor size.")
+       doc: /* Set cursor size.  */)
     (size)
     Lisp_Object size;
 {
@@ -759,40 +713,26 @@ DEFUN ("set-cursor-size", Fset_cursor_size, Sset_cursor_size, 1, 1, 0,
   cci.dwSize = XFASTINT (size);
   cci.bVisible = TRUE;
   (void) SetConsoleCursorInfo (cur_screen, &cci);
-  
+
   return Qt;
 }
 
-#ifndef HAVE_NTGUI
-void
-pixel_to_glyph_coords (struct frame * f, int pix_x, int pix_y, int *x, int *y,
-		      void *bounds, int noclip)
-{
-  *x = pix_x;
-  *y = pix_y;
-}
-
-void
-glyph_to_pixel_coords (struct window * f, int x, int y, int *pix_x, int *pix_y)
-{
-  *pix_x = x;
-  *pix_y = y;
-}
-#endif /* !HAVE_NTGUI */
-
 void
 syms_of_ntterm ()
 {
   DEFVAR_BOOL ("w32-use-full-screen-buffer",
                &w32_use_full_screen_buffer,
-  "Non-nil means make terminal frames use the full screen buffer dimensions.\n\
-This is desirable when running Emacs over telnet, and is the default.\n\
-A value of nil means use the current console window dimensions; this\n\
-may be preferrable when working directly at the console with a large\n\
-scroll-back buffer.");
-  w32_use_full_screen_buffer = 1;
+	       doc: /* Non-nil means make terminal frames use the full screen buffer dimensions.
+This is desirable when running Emacs over telnet.
+A value of nil means use the current console window dimensions; this
+may be preferrable when working directly at the console with a large
+scroll-back buffer.  */);
+  w32_use_full_screen_buffer = 0;
 
   defsubr (&Sset_screen_color);
   defsubr (&Sset_cursor_size);
   defsubr (&Sset_message_beep);
 }
+
+/* arch-tag: a390a07f-f661-42bc-aeb4-e6d8bf860337
+   (do not change this comment) */
